#include <stdio.h>
#include <SDL_gfxPrimitives.h>
#include "falak.h"

/** A lancolt listahoz szukseges pointerek */
FalLista *uj_fal;
FalLista *mozgoptr_fal;
FalLista *elso_fal;
FalLista *utolso_fal;

/** Fal strukturat visszado fuggveny *
  * Erre azert van szukseg, mert nem *
  * mindenutt hasznalhato a {}-es    *
  * struktura incializalas.          */
Fal Fal_legyen (int dir, int fix, int h1, int h2) {
    Fal f;
    f.touched = 0;
    f.dir = dir;
    if (dir == 1) {
        f.a = Pont_legyen(fix, h1);
        f.b = Pont_legyen(fix, h2);
    }
    else if (dir == 0) {
        f.a = Pont_legyen(h1, fix);
        f.b = Pont_legyen(h2, fix);
    }
    return f;
}

/** Letrehoz egy uj falat, azaz hozzadadja a parameterek *
  * altal meghatarozott falat a lancolt listahoz.        *
  * A falak csak vizszintes es fuggoleges helyzetuek     *
  * lehetnek, ennek garantalasara az oket letrehozo      *
  * fuggvenyek nem a ket vegpont koordinatait kerik be.  *
  * dir: a fal iranya: 0 - vizszintes, 1 - fuggoleges    *
  * fix: a pontok kozos koordinataja, vizszintes fal     *
  *      eseten y, fuggoleges eseten x koordinata lesz   *
  * h1, h2: a pontok nem kozos koordinatai               */
void Fal_letrehoz (int dir, int fix, int h1, int h2) {
    uj_fal = (FalLista*)malloc(sizeof(FalLista));
    uj_fal->fal = Fal_legyen(dir, fix, h1, h2);
    uj_fal->kov = NULL;
    if (elso_fal != NULL) {
        utolso_fal->kov = uj_fal;
    }
    else
        elso_fal = uj_fal;
    utolso_fal = uj_fal;
}

/** Debug fuggveny, egy fal ket *
  * vegpontjat irja ki a logba. */
void Fal_kiir (Fal f) {
    printf("x1: %d, y1: %d, x2: %d, y2: %d\n", f.a.x, f.a.y, f.b.x, f.b.y);
};

/** Debug fuggveny, a falak teljes    *
  * lancolt listajat irja ki a logba. */
void FalLista_kiir (void) {
    int i = 1;
    printf("Fallistat kezelo toltesek:\n");
    printf("\tuj %p\n\telso %p\n\tutolso%p\n", uj_fal, elso_fal, utolso_fal);
    printf("A falak lancolt listaja:\n");
    mozgoptr_fal = elso_fal;
    if (mozgoptr_fal == NULL)
        printf("\tA lista jelenleg ures.\n");
    while (mozgoptr_fal != NULL) {
        printf("\t%d. mozgoptr %p, kov %p   ", i, mozgoptr_fal, mozgoptr_fal->kov);
        Fal_kiir(mozgoptr_fal->fal);
        mozgoptr_fal = mozgoptr_fal->kov;
        i++;
    }
}

/** Felrajzol egy falat.                *
  * Ha a fallal eppen utkozik a proton, *
  * a falat pirossal rajzolja fel.      */
void Fal_rajzol (SDL_Surface *screen, Fal f) { // meg mindig egy kicsit atmegy a falakon, de az egy dolog
    if (f.touched)
        thickLineRGBA(screen, f.a.x, f.a.y, f.b.x, f.b.y, 2, 255, 0, 0, 200);
    else
        thickLineRGBA(screen, f.a.x, f.a.y, f.b.x, f.b.y, 2, 255, 255, 255, 200);
}

/** Ellenorzi, hogy a proton eppen hozzaer-e   *
  * barmelyik fuggoleges falhoz.               *
  * Visszateresi ertekek:                      *
  * 0 - Nem er hozza egy fuggoleges falhoz sem *
  * 1 - Balrol erint egy fuggoleges falat      *
  * 2 - Jobbrol erint egy fuggoleges falat     */
int fuggoleges_falhoz_er (Mozgotoltes p) {
    int returnvalue = 0;
    mozgoptr_fal = elso_fal;
    while (mozgoptr_fal != NULL) {
        if (mozgoptr_fal->fal.dir == 1) { /** Ha viszszintes a fal */
            if ((mozgoptr_fal->fal.a.y <= p.y && p.y <= mozgoptr_fal->fal.b.y && p.x - p.r <= mozgoptr_fal->fal.a.x && mozgoptr_fal->fal.a.x <= p.x + p.r) || /** Ha hozzaer a proton */
                (mozgoptr_fal->fal.b.y <= p.y && p.y <= mozgoptr_fal->fal.a.y && p.x - p.r <= mozgoptr_fal->fal.a.x && mozgoptr_fal->fal.a.x <= p.x + p.r)) {
                if (p.x <= mozgoptr_fal->fal.a.x  && p.v.vx > 0) { /** Melyik oldalrol erinti */
                    mozgoptr_fal->fal.touched = 1;
                    returnvalue = 1;
                }
                else if (p.x >= mozgoptr_fal->fal.a.x && p.v.vx < 0) {
                    mozgoptr_fal->fal.touched = 2;
                    returnvalue = 2;
                }
            }
            else
                mozgoptr_fal->fal.touched = 0;
        }
        mozgoptr_fal = mozgoptr_fal->kov;
    }
    return returnvalue;
}

/** Ellenorzi, hogy a proton eppen hozzaer-e   *
  * barmelyik vizszintes falhoz.               *
  * Visszateresi ertekek:                      *
  * 0 - Nem er hozza egy vizszintes falhoz sem *
  * 1 - Felulrol erint egy vizszintes falat    *
  * 2 - Alulrol erint egy vizszintes falat     */
int vizszintes_falhoz_er (Mozgotoltes p) {
    int returnvalue = 0;
    mozgoptr_fal = elso_fal;
    while (mozgoptr_fal != NULL) {
        if (mozgoptr_fal->fal.dir == 0) { /** Ha viszszintes a fal */
            if ((mozgoptr_fal->fal.a.x <= p.x && p.x <= mozgoptr_fal->fal.b.x && p.y - p.r <= mozgoptr_fal->fal.a.y && mozgoptr_fal->fal.a.y <= p.y + p.r) || /** Ha hozzaer a proton */
                (mozgoptr_fal->fal.b.x <= p.x && p.x <= mozgoptr_fal->fal.a.x && p.y - p.r <= mozgoptr_fal->fal.a.y && mozgoptr_fal->fal.a.y <= p.y + p.r)) {
                if (p.y <= mozgoptr_fal->fal.a.y  && p.v.vy > 0) { /** Melyik oldalrol erinti */
                    mozgoptr_fal->fal.touched = 1;
                    returnvalue = 1;
                }
                else if (p.y >= mozgoptr_fal->fal.a.y && p.v.vy < 0) {
                    mozgoptr_fal->fal.touched = 2;
                    returnvalue = 2;
                }
            }
            else
                mozgoptr_fal->fal.touched = 0;
        }
        mozgoptr_fal = mozgoptr_fal->kov;
    }
    return returnvalue;
}

/** Ez egy kulon fuggveny, ami akkor hivodik meg, *
  * amikor a felhasznalo toltest probal lerakni   */
int falban_lenne (int x, int y, int toltesr) {
    mozgoptr_fal = elso_fal;
    while (mozgoptr_fal != NULL) {
        if ((mozgoptr_fal->fal.a.x <= x && x <= mozgoptr_fal->fal.b.x && y - toltesr <= mozgoptr_fal->fal.a.y && mozgoptr_fal->fal.a.y <= y + toltesr) ||
            (mozgoptr_fal->fal.b.x <= x && x <= mozgoptr_fal->fal.a.x && y - toltesr <= mozgoptr_fal->fal.a.y && mozgoptr_fal->fal.a.y <= y + toltesr) ||
            (mozgoptr_fal->fal.a.y <= y && y <= mozgoptr_fal->fal.b.y && x - toltesr <= mozgoptr_fal->fal.a.x && mozgoptr_fal->fal.a.x <= x + toltesr) ||
            (mozgoptr_fal->fal.b.y <= y && y <= mozgoptr_fal->fal.a.y && x - toltesr <= mozgoptr_fal->fal.a.x && mozgoptr_fal->fal.a.x <= x + toltesr) ){
            return 1;
        }
        mozgoptr_fal = mozgoptr_fal->kov;
    }
    return 0;
}
