/* InfoC SDL project * http://infoc.eet.bme.hu/ */

#include <SDL.h>
#include <SDL_gfxPrimitives.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/** A kovetkezo fuggvenyek tobbszor *
    is egymast hivjak meg           */
void fatal_error (int errorcode);
void freememory (void);
void clearlevel (void);
void closeSDL(void);

#include "szinek.h"
#include "pontok.h"
#include "vektorok.h"
#include "toltesek.h"
#include "falak.h"
#include "menu.h"

/** Globalis valtozok */
int menuwidth = 200; /** Menu szelessege */
int WIDTH = -1; /** Ablak szelessege */
int HEIGHT = -1; /** Ablak magassaga*/
double t = 0.001; /** Kepfrissites ideje */

/** A program leallitasahoz hasznalt fuggvenyek */
void fatal_error (int errorcode){
    /** Hibauzenet a log fajlba */
    printf("\nVegzetes hiba tortent a program futasa kozben. (Hibakod: 0x%X)\n", errorcode);
    printf("A program leallitasa...\n");
    /** Memoria tisztitasa */
    freememory();
    /** SDL bezarasa */
    closeSDL();
    /** Log fajl lezarasa */
    printf("A program leallitasa sikeres volt.\n");
    printf("\n<--- LOG FAJL VEGE --->");
    exit(errorcode);
}

/** Torol mindent a palyarol (alaphelyzetbe *
  * allitja a lancolt listakat).            */
void clearlevel (void) {
    printf("Lancolt listak nullazasa...\n");
    /** Toltesek torlese */
    printf("\tToltesek torlese...\n");
    mozgoptr = elso;
    ToltesLista *torol;
    while (mozgoptr != NULL) {
        torol = mozgoptr;
        mozgoptr = mozgoptr->kov;
        free(torol);
    }
    elso = utolso = elsonemfix = utolsofix = NULL;
    printf("\tToltesek torolve.\n");
    /** Falak torlese */
    printf("\tFalak torlese...\n");
    mozgoptr_fal = elso_fal;
    FalLista *torol_fal;
    while (mozgoptr_fal != NULL) {
        torol_fal = mozgoptr_fal;
        mozgoptr_fal = mozgoptr_fal->kov;
        free(torol_fal);
    }
    elso_fal = utolso_fal = NULL;
    printf("\tFalak torolve.\n");
    printf("Lancolt listak alaphelyzetbe allitva.\n");
}

/** Az alabbi fuggvenyhez hasznalt pointerek */
int *fptr_sorok;
Pont **fptr_racspontmx;
/** Felszabaditja a racspontokat *
  * tartalmazo dinamikus tombot  *
  * es a lancolt listakat is.    */
void freememory (void) { //Aki malloc-ot mond, mondjon free-t is!
    int i;
    /** Dinamikus tomb torlese */
    if (fptr_racspontmx != NULL) {
    for (i = 0; i < *fptr_sorok; i++)
        free(fptr_racspontmx[i]);
    free(fptr_racspontmx);
    }
    /** Lancolt listak torlese */
    clearlevel();
}

/** Az alabbi fuggvenyhez hasznalt pointer */
SDL_TimerID *ptr_id1 = NULL;
/** Megallitja a futo idozitot, majd *
  * leallitja az SDL feluletet.      */
void closeSDL (void) {
    printf("SDL felulet bezarasa...\n");
    if (ptr_id1 != NULL) SDL_RemoveTimer(*ptr_id1);
    SDL_Quit();
    printf("SDL felulet bezarva.\n");
}

/** Palyak */
double map_posx;
double map_posy;
double map_vx;
double map_vy;
Pont map_celpont;

typedef struct {
    int score;
    int pausecounter;
    int resetcounter;
} Palyaadatok;

Palyaadatok levels[6] = {{0, 0, 0}, /** Ebbol az elembol csak a score-t hasznalja, az ossz. pontszam szamolasahoz*/
                         {0, 0, 0},
                         {0, 0, 0},
                         {0, 0, 0},
                         {0, 0, 0},
                         {0, 0, 0}};

/** Eldonti, hogy egy kattintas racspont kozeleben *
  * volt-e, visszairja a pointerekben atvett       *
  * integerekbe a racspont koordinatait.           */
int racspont_e (int x, int y, Pont **racspontmx, int sorok, int oszlopok, int *racsindexi, int *racsindexj, int toltesr) {
    int i, j;
    for (i = 0; i < sorok; i++)
        for (j = 0; j < oszlopok; j++)
            if(ponttav((double)racspontmx[i][j].x, (double)racspontmx[i][j].y, (double)x, (double)y) < toltesr) {
                *racsindexi = i;
                *racsindexj = j;
                return 1;
            }
    return 0;
}

/** Igaz erteket ad, ha a proton a celponthoz er */
int celpont_hit(Mozgotoltes proton, int celpontr) {
    return ponttav(proton.x, proton.y, (double)map_celpont.x, (double)map_celpont.y) < (proton.r + celpontr);
}

/** Kirajzolja a celpontot */
void celpont_rajzol (SDL_Surface *screen, int celpontr) {
    lineRGBA(screen, map_celpont.x-sqrt(2)/2*celpontr+1, map_celpont.y-sqrt(2)/2*celpontr+1, map_celpont.x+sqrt(2)/2*celpontr-1, map_celpont.y+sqrt(2)/2*celpontr-1, 255, 255, 255, 100);
    lineRGBA(screen, map_celpont.x-sqrt(2)/2*celpontr+1, map_celpont.y+sqrt(2)/2*celpontr-1, map_celpont.x+sqrt(2)/2*celpontr-1, map_celpont.y-sqrt(2)/2*celpontr+1, 255, 255, 255, 100);
    filledCircleRGBA(screen, map_celpont.x, map_celpont.y, celpontr, 255, 255, 255, 80);
    aacircleRGBA(screen, map_celpont.x, map_celpont.y, celpontr, 255, 255, 255, 127);
}

/** A toplista elemeinek strukturaja, *
  * egy nevet es a hozza tartozo      *
  * pontszamot tartalmazza            */
typedef struct {
    int score;
    char name[16+1];
} highscore_entry;

/** Beolvassa a highscores.txt *
  * fajlbol a dicsoseglistat   */
void highscores_read (highscore_entry *highscore_list, int hsdb) {
    FILE *fscores;
    fscores = fopen("highscores.txt", "rt");
    if (fscores == NULL) {
        printf("Nem sikerult megnyitni a toplista fajlt olvasasra!\n");
        fatal_error(0xF1);
    }
    int i, j;
    int inputscore;
    char inputname[16+1];
    printf("Toplista fajl olvasasa...\n");
    /** Beolvas annyi sort, amennyit a jatek a toplistaba var */
    for (i = 0; i <= hsdb; i++) {
        fscanf(fscores, "%d;%s", &inputscore, inputname);
        for (j = 0; inputname[j] != '\0'; j++)
            highscore_list[i].name[j] = inputname[j];
        highscore_list[i].name[j] = '\0';
        highscore_list[i].score = inputscore;
        printf("%2d.: %s - %d\n", i, inputname, inputscore);
    }
    /** Beolvas meg egy sort, aminek a fajlt kell lezarnia */
    fscanf(fscores, "%d;%s", &inputscore, inputname);
    if (inputscore != 0 || inputname[0] != '0') {
        printf("Hibas volt a toplista fajl - hianyzott a faljt lezaro sor!\n");
        fatal_error(0xF1);
    }
    printf("Toplista fajl beolvasva.\n\n");
    fclose(fscores);
}

/** Visszairja a dicsoseglistat a fajlba */
void highscores_write (highscore_entry *highscore_list, int hsdb) {
    FILE *fscores;
    fscores = fopen("highscores.txt", "wt");
    if (fscores == NULL) {
        printf("Nem sikerult megnyitni a toplista fajlt irasra!\n");
        fatal_error(0xF2);
    }
    int i;
    printf("\nToplista faljba irasa...\n");

    /** Elokesziti a fajl elejet */
    highscore_list[0].score = 0;
    highscore_list[0].name[0] = '0';
    highscore_list[0].name[1] = '\0';

    /** Fajlba irja a dicsoseglistat */
    for (i = 0; i <= hsdb; i++)
        fprintf(fscores, "%d;%s\n", highscore_list[i].score, highscore_list[i].name);
    fprintf(fscores, "0;0\n");
    printf("A toplista fajlba irasa sikeresen megtortent.\n\n");
    fclose(fscores);
}

/** Sorba rendezi a dicsoseglista rekordjait */
void highscores_sort (highscore_entry *highscore_list, int hsdb) {
    int i, j;
    int min = 0;
    for (i = 0; i <= hsdb; i++) /** Legrosszabb kidobasa minimumkivalasztassal */
        if (highscore_list[i].score < highscore_list[min].score)
            min = i;
    if (min != 0) {
        highscore_entry csere = highscore_list[0];
        highscore_list[0] = highscore_list[min];
        highscore_list[min] = csere;
    }
    highscore_list[0].name[0] = '\0'; /** Nev nullazasa, hogy utana ebbe a stringbe lehessen olvasni az uj nevet */

    int max;
    for (i = 1; i <= hsdb; i++) { /** A maradek rendezese maximumkivalasztassal */
        max = i;
        for (j =i + 1; j <= hsdb; j++) {
            if (highscore_list[j].score > highscore_list[max].score)
                max = j;
        }
        if (max != i) {
            highscore_entry csere = highscore_list[i];
            highscore_list[i] = highscore_list[max];
            highscore_list[max] = csere;
        }
    }
}

/** Standard SDL idozito */
Uint32 idozit (Uint32 ms, void *param) {
    SDL_Event ev;
    ev.type = SDL_USEREVENT;
    SDL_PushEvent(&ev);
    return ms;
}

/** Betolt egy adott sorszamu palyat */
void loadLevel (int lvl, Mozgotoltes *proton) {
    printf("%d. palya betoltese...\n", lvl);
    clearlevel(); /** Torli az elozo palyat */

    FILE *flevels;
    flevels = fopen("levels.txt", "rt"); /** Megnyitja a palyakat tartalmazo fajlt */
    if (flevels == NULL) {
        printf("Nem sikerult megnyitni a palyakat tartalmazo fajlt olvasasra!\n");
        fatal_error(0xF3);
    }
    char lvlnumber = lvl + '0';

    if (WIDTH == -1) { /** Ha eloszor hivodik meg a fuggveny, beolvassa a fajl tetejerol a jatekablak meretet */
        fscanf(flevels, "%d;%d", &WIDTH, &HEIGHT); /** Palya mereteinek beolvasasa */
        if (WIDTH == -1 || HEIGHT == -1) {
            printf("Hiba a jatekablak mereteinek beolvasasa kozben!\n");
            fatal_error(0xF4);
        }
        /** Egy kis minimum-maximum korrigalas, hogy ne lehessen barmekkora az ablak */
        WIDTH += menuwidth;
        if (HEIGHT < 550) HEIGHT = 550;
        if (WIDTH > 1920) WIDTH = 1920;
        if (HEIGHT > 1080) HEIGHT = 1080;
   }

    char in[100];
    do { /** Addig keres a faljban, amig megtalalja a keresett palya elejet jelzo sort */
        fscanf(flevels, "%s", in);
    } while(in[0] != 'B' || in[1] != lvlnumber);
    /** Beolvassa az alap adatokat, beallitja a proton helyzetet */
    fscanf(flevels, "%d,%d:cp", &map_celpont.x, &map_celpont.y);
    fscanf(flevels, "%lf:px", &map_posx); proton->x = map_posx;
    fscanf(flevels, "%lf:py", &map_posy); proton->y = map_posy;
    fscanf(flevels, "%lf:vx", &map_vx); proton->v.vx = map_vx;
    fscanf(flevels, "%lf:vy", &map_vy); proton->v.vy = map_vy;
    /** Kiirja az alapadatokat a logba, ellenorzes keppen */
    printf("Palya adatok:\n");
    printf("\tcelpont (x, y)\t%d,%d\n", map_celpont.x, map_celpont.y);
    printf("\tproton (x, y)\t%.2f, %.2f\n", map_posx, map_posy);
    printf("\tx sebesseg\t%.4f\n", proton->v.vx);
    printf("\ty sebesseg\t%.4f\n", proton->v.vy);

    char flag;
    fscanf(flevels, " %c", &flag);
    if (flag == 'F') {
        printf("Vannak falak a palyan\n");
        /** Falakat keres */
        int dir, fix, h1, h2;
        fscanf(flevels, " %c,%d,%d,%d,%d", &flag, &dir, &fix, &h1, &h2);
        while (flag == 'F') { /** Amig falat talal */
            Fal_letrehoz(dir, fix, h1, h2);
            printf("\tFal: %d %d %d %d\n", dir, fix, h1, h2);
            fscanf(flevels, " %c,%d,%d,%d,%d", &flag, &dir, &fix, &h1, &h2);
        }
    }
    if (flag == 'T') {
        printf("Vannak toltesek a palyan\n");
        /** Tolteseket keres */
        int x, y, charge;
        fscanf(flevels, " %c,%d,%d,%d", &flag, &x, &y, &charge);
        while (flag == 'T') { /** Amig toltest talal */
            fixToltes_letrehoz(x, y, charge);
            printf("\tToltes: %d %d %d \n", x, y, charge);
            fscanf(flevels, " %c,%d,%d,%d", &flag, &x, &y, &charge);
        }
    }

    fclose(flevels); /** Becsukja a fajlt */
    ToltesLista_kiir(); /** Logba irja a toltesek listajat */
    FalLista_kiir(); /** Logba irja a falak listajat */

    if (flag == 'E') /** Ha elert az "E" jelhez, a beolvasas rendben volt */
        printf("%d. palya betoltve!\n", lvl);
    else {
        printf("Hiba tortent a %d. palya betoltese kozben!\n", lvl);
        fatal_error(0xF4);
    }
    printf("\n");
}

/** A dicsoseglista kerpernyo megjeleniteset vegzi */
void frissit_hs (SDL_Surface *screen, highscore_entry *highscore_list, int hsdb, int textinput, int *endoftextinput, Button *menu) {
    int i;
    int entrywidth = 200; /** A rekordok kis teglalalpjainak szelessege */
    int topoffset = 50; /** A nagy teglalap tavolsaga a kerpernyo tetejetol*/
    int padding = 10; /** A rekordok kis teglalapjai koruli "margo" */
    int currentoffset = 20;
    char scoretext[6]; /** A pontszamok kiirasahoz hasznalt ideiglenes string */

    /** A nagy teglalap megrajzolasa */
    boxRGBA(screen, menuwidth, 0, WIDTH, HEIGHT, accent.mapcolor.r, accent.mapcolor.g, accent.mapcolor.b, 255);
    boxRGBA(screen, (WIDTH+menuwidth)/2 - entrywidth/2 - padding, 1*10 + topoffset - padding, (WIDTH+menuwidth)/2 + entrywidth/2 + padding, hsdb*10 + hsdb*30 + topoffset + padding, accent.menubgcolor.r, accent.menubgcolor.g, accent.menubgcolor.b, 255);

    for (i = 1; i <= hsdb; i++) { /** A lista elemeinek kirajzolasa es kiirasa */
        boxRGBA(screen, (WIDTH+menuwidth)/2 - entrywidth/2, i*10 + (i-1)*30 + topoffset, (WIDTH+menuwidth)/2 + entrywidth/2, i*10 + i*30 + topoffset, accent.btenabled.r, accent.btenabled.g, accent.btenabled.b, 255);
        stringRGBA(screen, (WIDTH+menuwidth)/2 - entrywidth/2 + 10, i*10 + (i-1)*30 + topoffset + 12, highscore_list[i].name, accent.textenabled.r, accent.textenabled.g, accent.textenabled.b, 255);
        sprintf(scoretext, "%5d", highscore_list[i].score);
        stringRGBA(screen, (WIDTH+menuwidth)/2 + entrywidth/2 - 10 - (5*8), i*10 + (i-1)*30 + topoffset + 12, scoretext, accent.textenabled.r, accent.textenabled.g, accent.textenabled.b, 255);
    }

    if (textinput) { /** A szovegbeviteli mezo kirajzolasa */
        i++;
        boxRGBA(screen, (WIDTH+menuwidth)/2 - entrywidth/2 - padding, (hsdb+1)*10 + hsdb*30 + topoffset - padding + currentoffset, (WIDTH+menuwidth)/2 + entrywidth/2 + padding, (hsdb+1)*10 + (hsdb+1)*30 + topoffset + padding + currentoffset, accent.menubgcolor.r, accent.menubgcolor.g, accent.menubgcolor.b, 255);
        boxRGBA(screen, (WIDTH+menuwidth)/2 - entrywidth/2, (hsdb+1)*10 + hsdb*30 + topoffset + currentoffset, (WIDTH+menuwidth)/2 + entrywidth/2, (hsdb+1)*10 + (hsdb+1)*30 + topoffset + currentoffset, accent.btenabled.r, accent.btenabled.g, accent.btenabled.b, 255);
        stringRGBA(screen, (WIDTH+menuwidth)/2 - entrywidth/2 + 10, (hsdb+1)*10 + hsdb*30 + topoffset + 12 + currentoffset, highscore_list[0].name, accent.textenabled.r, accent.textenabled.g, accent.textenabled.b, 255);
        sprintf(scoretext, "%5d", highscore_list[0].score);
        stringRGBA(screen, (WIDTH+menuwidth)/2 + entrywidth/2 - 10 - (5*8), (hsdb+1)*10 + hsdb*30 + topoffset + 12 + currentoffset, scoretext, accent.textenabled.r, accent.textenabled.g, accent.textenabled.b, 255);
        stringRGBA(screen, (WIDTH+menuwidth)/2 - entrywidth/2 - 25,(hsdb+1)*10 + hsdb*30 + topoffset + 12 + currentoffset + 38, "ENTER YOUR NAME AND PRESS ENTER", accent.textenabled.r, accent.textenabled.g, accent.textenabled.b, 255);
    }
    /** Rendezes */
    if (*endoftextinput) { /** Ha rendezni kell (ENTER lenyomasa utan) */
        highscores_sort(highscore_list, hsdb);
        *endoftextinput = 0;
    }
    /** Menu */
    drawmenu(screen, menu);
    SDL_Flip(screen);
}

/** A palya sikeres teljesitese utan a kerpernyot frissito fuggveny */
void frissit_levelend (SDL_Surface *screen, int *start, int rstcounter, int pausecounter, int current, Button *menu, int bt_level1, int *highscoreclick, int buttonwidth, highscore_entry *highscore_list) {
    *start = -1;
    boxRGBA(screen, menuwidth, 0, WIDTH, HEIGHT, 0, 0, 0, 75);
    char scoretext[18];
    if (levels[current].score == 0) {
        levels[current].score = (int)fmax(5000-pausecounter*750-rstcounter*500, 0);
        levels[0].score += levels[current].score;
    }
    stringRGBA(screen, (WIDTH+menuwidth)/2-75+12, HEIGHT/2-23, "CONGRATULATIONS!", accent.textcolor.r, accent.textcolor.g, accent.textcolor.b, 255);
    sprintf(scoretext, "LEVEL %d SCORE: %d", current, levels[current].score); /** Pontszam kiirasa */
    stringRGBA(screen, (WIDTH+menuwidth)/2-75, HEIGHT/2-10, scoretext, accent.textcolor.r, accent.textcolor.g, accent.textcolor.b, 255);

    drawmenu(screen, menu); /** Menu ujrarajzolasa */

    if (current == 5) { /** Jatek vege, dicsoseglista gomb rajzolasa a kepernyore */
        boxRGBA(screen, (WIDTH+menuwidth)/2 - buttonwidth/2, HEIGHT/2+10, (WIDTH+menuwidth)/2 + buttonwidth/2, HEIGHT/2+10 + 30, accent.btnew.r, accent.btnew.g, accent.btnew.b, 255);
        stringRGBA(screen, (WIDTH+menuwidth)/2 - buttonwidth/2 + 10, HEIGHT/2+10 + 12, "CLICK HERE TO ENTER YOUR NAME", accent.textnew.r, accent.textnew.g, accent.textnew.b, 255);
        *highscoreclick = 1;
        highscore_list[0].score = levels[0].score;
        sprintf(scoretext, "TOTAL SCORE: %d", levels[0].score);
        stringRGBA(screen, (WIDTH+menuwidth)/2-67, HEIGHT/2+22+32, scoretext, accent.textcolor.r, accent.textcolor.g, accent.textcolor.b, 255);
    }
    else { /** Kovetkezo palya feloldasa */
        int i;
        for (i = bt_level1; i <= bt_level1 + current; i++)
        if (!menu[i].enabled)
            menu[i].enabled = -1;
    }
    SDL_Flip(screen);
}

void frissit (SDL_Surface *screen, int beosztas, int elsooszlop, int toltesr, Mozgotoltes proton, int rstcounter, int pausecounter, Button *menu, int bt_shownumbers, int celpontr) {
    boxRGBA(screen, 0, 0, WIDTH, HEIGHT, accent.mapcolor.r, accent.mapcolor.g, accent.mapcolor.b, 255);
    int i;
     /** Fuggoleges racsvonalak*/
    for (i = beosztas; i < HEIGHT; i += beosztas)
        lineRGBA(screen, menuwidth+1, i, WIDTH, i, 255, 255, 255, 50);
    /** Vizszintes racsvonalak */
    for (i = elsooszlop; i < WIDTH; i += beosztas)
        lineRGBA(screen, i, 0, i, HEIGHT, 255, 255, 255, 50);
    /** Toltesek */
    mozgoptr = elso;
    while (mozgoptr != NULL) {
        Toltes_rajzol(screen, mozgoptr->toltes, toltesr, proton, menu[bt_shownumbers].clicked);
        mozgoptr = mozgoptr->kov;
    }
    /** Falak */
    mozgoptr_fal = elso_fal;
    while (mozgoptr_fal != NULL) {
        Fal_rajzol(screen, mozgoptr_fal->fal);
        mozgoptr_fal = mozgoptr_fal->kov;
    }
    /** Mozgotoltes */
    Mozgotoltes_rajzol(screen, proton);
    /** Celpont */
    celpont_rajzol(screen, celpontr);
    /** Menu */
    drawmenu(screen, menu);
    /** Szamlalok a bal also sarokba */
    char pausetext[12];
    sprintf(pausetext, "PAUSE: %d", pausecounter);
    stringRGBA(screen, 10, HEIGHT-39, pausetext, accent.textcolor.r, accent.textcolor.g, accent.textcolor.b, 200);
    char resettext[12];
    sprintf(resettext, "RESET: %d", rstcounter);
    stringRGBA(screen, 10, HEIGHT-52, resettext,accent.textcolor.r, accent.textcolor.g, accent.textcolor.b, 200);
    char totalscore[20];
    sprintf(totalscore, "TOTAL SCORE: %d", levels[0].score);
    stringRGBA(screen, 10, HEIGHT-13, totalscore, accent.textcolor.r, accent.textcolor.g, accent.textcolor.b, 200);
    SDL_Flip(screen);
}

int main (int argc, char *argv[]) {
    printf("<--- LOG FAJL ELEJE --->\n\n");

    /** A proton letrehozasa */
    Mozgotoltes proton;
    proton.charge = 2;
    proton.r = 8;

    int celpontr = 10; /** Celpont sugara */
    int toltesr = 10; /** Toltesek sugara */
    Vektor eredovektor = {0, 0}; /** Eredovektor a mozgashoz */
    double vmax = 1; /** Sebesseg maximumerteke */

    /** Az elso palya betoltese a program inditasakor */
    loadLevel(1, &proton);
    /** Alap szinek */
    int colorschemeid = 0; /** A szinsemak valtasahoz */
    int numberofschemes = 4; /** A jatekban talalhato szinsemak szama */
    int reloadcolors = 1; /** Ujratoltesre kerul a szinsema, ha az erteke 1 */
    srand(time(0)); /** Random szinek generalasahoz */

    int i, j; /* Ciklusvaltozok */
    /** Racs */
    int beosztas = 50; /** Legyen beosztas > toltesr ! */
    int racsindexi, racsindexj; /** Ezeket a valtozokat hasznalja a racspont_e fuggveny kimenetkent */
    int sorok = 0, oszlopok = 0;
    /** Racs meretenek kiszamolasa */
    for (i = menuwidth+beosztas; i % beosztas != 0; i++) {
        /*ures ciklus*/
    }
    int elsooszlop = i;
    for (i = beosztas; i < HEIGHT; i += beosztas)
        sorok++;
    for (j = elsooszlop; j < WIDTH; j += beosztas)
        oszlopok++;
    /** Malloc */
    Pont **racspontmx;
    racspontmx = (Pont**) malloc(sorok * sizeof(Pont*));
    fptr_sorok = &sorok; /** Ertekadas a globalis pointernek, felszabaditashoz */
    fptr_racspontmx = racspontmx; /** Ertekadas a globalis pointernek, felszabaditashoz */
    if (racspontmx == NULL) {
        printf("FATAL ERROR: Nem sikerult memoriat foglalni a racspontok sorainak!");
        fatal_error(0xA3);
    }
    for (i = 0; i < sorok; i++) {
        racspontmx[i] = (Pont*) malloc(oszlopok * sizeof(Pont));
        if (racspontmx[i] == NULL) {
            printf("FATAL ERROR: Nem sikerult memoriat foglalni a racspontok oszlopainak!");
            fatal_error(0xA4);
        }
    }
    /** Racsponttomb feltoltese */
    for (i = beosztas, racsindexi = 0; i < HEIGHT; i += beosztas, racsindexi++)
        for (j = elsooszlop, racsindexj = 0; j < WIDTH; j += beosztas, racsindexj++)
            racspontmx[racsindexi][racsindexj] = Pont_legyen(j, i);
    racsindexi = racsindexj = 0;

    /** Menu konfiguralasa */
    Button menu[15];
    enum {
        underflow = 0,
        bt_level1 = 1,
        bt_level2 = 2,
        bt_level3 = 3,
        bt_level4 = 4,
        bt_level5 = 5,
        bt_start = 6,
        bt_protonreset = 7,
        bt_mapreset = 8,
        bt_shownumbers = 9,
        bt_highscores = 10,
        bt_accentcolor = 11,
        bt_exit = 12,
        overflow = 13
    };

    Button buf = {-1, -1, "underflow", -1};     menu[underflow] = buf;
    Button bss = {0, 1, "_START / STOP", 1};    menu[bt_start] = bss;
    Button brp = {0, 1, "_RESET PROTON", 0};    menu[bt_protonreset] = brp;
    Button brm = {0, 1, "RESET _MAP", 0};       menu[bt_mapreset] = brm;
    Button bsn = {1, 1, "SHOW _NUMBERS", 1};    menu[bt_shownumbers] = bsn;
    Button bac = {0, 1, "_THEME: PLCHLDR", 0};  menu[bt_accentcolor] = bac; /* A PLCHLDR szoveg sosem kerul megjelenitesre, helyere a szinsema neve kerul majd */
    Button bl1 = {0, 1, "LEVEL 1", 0};          menu[bt_level1] = bl1;
    Button bl2 = {0, 0, "LEVEL 2", 0};          menu[bt_level2] = bl2;
    Button bl3 = {0, 0, "LEVEL 3", 0};          menu[bt_level3] = bl3;
    Button bl4 = {0, 0, "LEVEL 4", 0};          menu[bt_level4] = bl4;
    Button bl5 = {0, 0, "LEVEL 5", 0};          menu[bt_level5] = bl5;
    Button bhs = {0, 1, "_HIGH SCORES", 0};     menu[bt_highscores] = bhs;
    Button bqg = {0, 1, "_QUIT GAME", 0};       menu[bt_exit] = bqg;
    Button bof = {-1, -1, "overflow", -1};      menu[overflow] = bof;           /** Zarjel */

    /** Dicsoseglista */
    int hs_clickable = 0; /** Az otodik palya utan a gomb megjelenesekor az erteke 1 lesz */
    int hs_buttonwidth = 254; /** Az otodik palya utan megjeleno gomb szelessege */
    int const hsdb = 10; /** A tarolt rekordok szama */
    highscore_entry highscore_list[hsdb + 1]; /** A rekordok tombje */
    highscores_read(highscore_list, hsdb); /** Fajlbol olvasas */
    highscores_sort(highscore_list, hsdb); /** Rendezes */
    int textinput = 0, endoftextinput = 0, text_cntr = 0; /** Valtozok a billentyuzet kezelesehez */

    /** SDL valtozok */
    SDL_Event event;
    SDL_Surface *screen;
    SDL_TimerID id1;
    ptr_id1 = &id1;
    /** SDL inicializalasa */
    SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
    screen=SDL_SetVideoMode(WIDTH+1, HEIGHT+1, 0, SDL_ANYFORMAT);
    if (!screen) {
        printf("Nem sikerult megnyitni az SDL feluletet!\n");
        fatal_error(0x01);
    }
    SDL_WM_SetCaption("GAME_WINDOW", "GAME_WINDOW");

    int quit = 0;
    int start = 0;
    int protonreset = 0;
    int mapreset = 0;
    int changelevel = 0, currentlevel = 1;
    while (!quit) {
        if (protonreset) { /** Proton alaphelyzetbe allitasa */
            start = -1;
            menu[bt_start].enabled = 1;
            menu[bt_mapreset].enabled = 1;
            levels[currentlevel].resetcounter++;
            proton.x = map_posx;
            proton.y = map_posy;
            proton.v.vx = map_vx;
            proton.v.vy = map_vy;
            eredovektor.vx = 0;
            eredovektor.vy = 0;
            protonreset = 0;
            SDL_Flip(screen); /* Hogy a proton mindig az uj helyen latszodjon */
        }
        if (mapreset) { /** Palya alaphelyzetbe allitasa */
            changelevel = currentlevel;
            mapreset = 0;
        }
        if (1 <= changelevel && changelevel <= 5 && menu[changelevel].enabled) { /** Palya betoltes */
            loadLevel (changelevel, &proton);
            currentlevel = changelevel;
            if (menu[changelevel].enabled == -1) /** Epp feloldott palyara kattintas kezelese */
                menu[changelevel].enabled = 1;
            menu[bt_start].enabled = 1;
            menu[bt_protonreset].enabled = 1;
            menu[bt_mapreset].enabled = 1;
            menu[bt_shownumbers].enabled = 1;
            start = -1;
            changelevel = 0;
        }
        if (start == -1) { /** Stop */
            menu[bt_start].clicked = 0;
            SDL_RemoveTimer(id1);
            start = 0;
        }
        else if (start == 1) { /** Start */
            start = 0;
            id1 = SDL_AddTimer(t*1000, idozit, NULL); /** A t valtozo masodpercben van megadva, ez a fuggveny *
                                                        * viszont ezredmasodpercet kap parameterkent          */
        }
        if (reloadcolors) { /** Tema ujratoltes */
            menu[bt_accentcolor].label[strlen("_THEME: ")] = '\0';
            switch (colorschemeid) {
                case 0: setaccent(grey); strcat(menu[bt_accentcolor].label, "CLASSIC"); break;
                case 1: setaccent(blue); strcat(menu[bt_accentcolor].label, "BLUE"); break;
                case 2: setaccent(green); strcat(menu[bt_accentcolor].label, "GREEN"); break;
                case 3: setaccent(red); strcat(menu[bt_accentcolor].label, "RED"); break;
                case 4: setaccent(randomscheme); strcat(menu[bt_accentcolor].label, "RANDOM"); break;
                default: setaccent(grey); strcat(menu[bt_accentcolor].label, "CLASSIC"); break;
            }
            reloadcolors = 0;
        }

        SDL_WaitEvent(&event); /** Varakozas */
        switch (event.type) {
        case SDL_MOUSEBUTTONDOWN:
            if (event.button.button == SDL_BUTTON_LEFT) {
                /** Start gomb */
                if (Button_click(bt_start, event.button.x, event.button.y, menu)) {
                    if (menu[bt_start].clicked) {
                        menu[bt_start].clicked = 0;
                        start = -1;
                        levels[currentlevel].pausecounter++;
                    }
                    else if (!menu[bt_start].clicked) {
                        menu[bt_start].clicked = 1;
                        start = 1;
                    }
                }
                /** Proton reset gomb */
                else if (Button_click(bt_protonreset, event.button.x, event.button.y, menu)) {
                    menu[bt_protonreset].clicked = 1;
                    protonreset = 1;
                }
                /** Map reset gomb */
                else if (Button_click(bt_mapreset, event.button.x, event.button.y, menu)) {
                    menu[bt_mapreset].clicked = 1;
                    mapreset = 1;
                    protonreset = 1;
                }
                /** Toggle numbers gomb */
                else if (Button_click(bt_shownumbers, event.button.x, event.button.y, menu)) {
                    if (menu[bt_shownumbers].clicked) {
                        menu[bt_shownumbers].clicked = 0;
                    }
                    else if (!menu[bt_shownumbers].clicked) {
                        menu[bt_shownumbers].clicked = 1;
                    }
                }
                /** Szinek gomb*/
                else if (Button_click(bt_accentcolor, event.button.x, event.button.y, menu)) {
                    menu[bt_accentcolor].clicked = 1;
                    if (colorschemeid < numberofschemes)
                        colorschemeid++;
                    else
                        colorschemeid = 0;
                    reloadcolors = 1;
                }
                /** Dicsoseglista gomb */
                else if (Button_click(bt_highscores, event.button.x, event.button.y, menu)) {
                    menu[bt_highscores].clicked = 1;
                }
                /** Dicsoseglista hozzaadas gomb (5. palya utan, a jatekteren, nem a menuben) */
                else if (((WIDTH+menuwidth)/2 - hs_buttonwidth/2 < event.button.x) && (event.button.x < (WIDTH+menuwidth)/2 + hs_buttonwidth/2) &&
                        (HEIGHT/2+10 < event.button.y) && (event.button.y < HEIGHT/2+10+30) && (hs_clickable == 1)) {
                        changelevel = -1;
                        textinput = 1;
                        hs_clickable = 0;
                        menu[bt_start].enabled = 0;
                        menu[bt_protonreset].enabled = 0;
                        menu[bt_mapreset].enabled = 0;
                        menu[bt_shownumbers].enabled = 0;
                }
                /** Kilepes gomb */
                else if (Button_click(bt_exit, event.button.x, event.button.y, menu)) {
                    menu[bt_exit].clicked = 1;
                }
                /** Level gombok */
                else {
                    for (i = bt_level1; i <= bt_level5; i++)
                        if (Button_click(i, event.button.x, event.button.y, menu))
                            menu[i].clicked = 1;
                }
            }
            /** Racspontra kattintas */
            if (!menu[bt_start].clicked &&
                racspont_e(event.button.x, event.button.y, racspontmx, sorok, oszlopok, &racsindexi, &racsindexj, toltesr) &&
                !(racspontmx[racsindexi][racsindexj].x == map_celpont.x && racspontmx[racsindexi][racsindexj].y == map_celpont.y) &&
                !falban_lenne(racspontmx[racsindexi][racsindexj].x, racspontmx[racsindexi][racsindexj].y, toltesr)) {
                printf("A jatekos racspontra kattintott: %d %d\n", racspontmx[racsindexi][racsindexj].x, racspontmx[racsindexi][racsindexj].y);
                /** Bal egergomb */
                if (event.button.button == SDL_BUTTON_LEFT) {
                    Toltes_novel(racspontmx[racsindexi][racsindexj]);
                }
                /** Jobb egergomb */
                else if (event.button.button == SDL_BUTTON_RIGHT) {
                    Toltes_csokkent(racspontmx[racsindexi][racsindexj]);
                }
                printf("\n");
            }
            break;
        case SDL_MOUSEBUTTONUP: /** Egergomb felengedesere torteno akciok */
            if (event.button.button == SDL_BUTTON_LEFT) {
                /** Kilepes */
                if (Button_click(bt_exit, event.button.x, event.button.y, menu) && menu[bt_exit].clicked) {
                    quit = 1;
                }
                /** Dicsoseglista gomb */
                else if (Button_click(bt_highscores, event.button.x, event.button.y, menu) && menu[bt_highscores].clicked) {
                    menu[bt_start].enabled = 0;
                    menu[bt_protonreset].enabled = 0;
                    menu[bt_mapreset].enabled = 0;
                    menu[bt_shownumbers].enabled = 0;
                    changelevel = -1;
                }
                /** Level gombok */
                else {
                    for (i = bt_level1; i <= bt_level5; i++)
                        if (Button_click(i, event.button.x, event.button.y, menu) && menu[i].clicked == 1)
                            changelevel = i;
                }
                /** Gombok felengedese */
                for (i = 1; menu[i].clicked != -1; i++)
                    if (menu[i].toggle == 0) {
                        menu[i].clicked = 0;
                    }
            }
            break;
        case SDL_KEYDOWN: /** Gomb lenyomas */
            if (textinput) { /** Szovegbevitel a dicsoseglista kerpernyon */
                if (event.key.keysym.sym == SDLK_BACKSPACE && text_cntr > 0) { /** BACKSPACE */
                    highscore_list[0].name[--text_cntr] = '\0';
                }
                else if (event.key.keysym.sym == SDLK_RETURN || event.key.keysym.sym == SDLK_KP_ENTER) { /** ENTER */
                    textinput = 0;
                    endoftextinput = 1;
                }
                if (text_cntr < 16) {
                    char tempstr[2];
                    if (97 <= event.key.keysym.sym && event.key.keysym.sym <= 122) { /** A - Z */
                        sprintf(tempstr, "%c", event.key.keysym.sym - 'a' + 'A');
                        strcat(highscore_list[0].name, tempstr);
                        text_cntr++;
                    }
                    else if (48 <= event.key.keysym.sym && event.key.keysym.sym <= 57) { /** 0 - 9 */
                        sprintf(tempstr, "%c", event.key.keysym.sym);
                        strcat(highscore_list[0].name, tempstr);
                        text_cntr++;
                    }
                    else if (256 <= event.key.keysym.sym && event.key.keysym.sym <= 265) { /** NUMPAD 0 - 9 */
                        sprintf(tempstr, "%c", event.key.keysym.sym - 256 + '0');
                        strcat(highscore_list[0].name, tempstr);
                        text_cntr++;
                    }
                    else if (event.key.keysym.sym == SDLK_SPACE) { /** SPACE */
                        strcat(highscore_list[0].name, " ");
                        text_cntr++;
                    }
                }
            }
            else { /** Gyorsgombok, normal hasznalatnal */ //if (!textinput)
                if ((event.key.keysym.sym == SDLK_q || event.key.keysym.sym == SDLK_ESCAPE) && menu[bt_exit].enabled) { /** Q, ESCAPE*/
                    menu[bt_exit].clicked = 1;
                }
                else if (event.key.keysym.sym == SDLK_r && menu[bt_protonreset].enabled) { /** R */
                    menu[bt_protonreset].clicked = 1;
                    protonreset = 1;
                }
                else if (event.key.keysym.sym == SDLK_m && menu[bt_mapreset].enabled) { /** M */
                    menu[bt_mapreset].clicked = 1;
                    mapreset = 1;
                    protonreset = 1;
                }
                else if (event.key.keysym.sym == SDLK_n && menu[bt_shownumbers].enabled) { /** N */
                    if (menu[bt_shownumbers].clicked) {
                        menu[bt_shownumbers].clicked = 0;
                    }
                    else if (!menu[bt_shownumbers].clicked) {
                        menu[bt_shownumbers].clicked = 1;
                    }
                }
                else if ((event.key.keysym.sym == SDLK_s || event.key.keysym.sym == SDLK_SPACE) && menu[bt_start].enabled) { /** S, SPACE */
                    if (menu[bt_start].clicked) {
                        menu[bt_start].clicked = 0;
                        start = -1;
                        levels[currentlevel].pausecounter++;
                    }
                    else if (!menu[bt_start].clicked) {
                        menu[bt_start].clicked = 1;
                        start = 1;
                    }
                }
                else if (event.key.keysym.sym == SDLK_t && menu[bt_accentcolor].enabled) { /** T */
                    menu[bt_accentcolor].clicked = 1;
                    if (colorschemeid < numberofschemes)
                        colorschemeid++;
                    else
                        colorschemeid = 0;
                    reloadcolors = 1;
                }
                else if (event.key.keysym.sym == SDLK_h) { /** H */
                    menu[bt_highscores].clicked = 1;
                    menu[bt_start].enabled = 0;
                    menu[bt_protonreset].enabled = 0;
                    menu[bt_mapreset].enabled = 0;
                    menu[bt_shownumbers].enabled = 0;
                    changelevel = -1;
                }
                else if (event.key.keysym.sym == SDLK_KP_PLUS) { /** Cheat */
                    for (i = bt_level1; i <= bt_level5; i++)
                        menu[i].enabled = -1;
                }
                else {
                    for (i = SDLK_1; i < SDLK_1 + 5; i++) /** 1- 5 */
                        if (event.key.keysym.sym == i) {
                            changelevel = i-SDLK_1+1;
                            menu[i-SDLK_1+1].clicked = 1;
                            break;
                        }
                    for (i = SDLK_KP1; i < SDLK_KP1 + 5; i++) /** NUMPAD 1-5 */
                        if (event.key.keysym.sym == i) {
                            changelevel = i-SDLK_KP1+1;
                            menu[i-SDLK_KP1+1].clicked = 1;
                            break;
                        }
                }
            }
            break;
        case SDL_KEYUP: /** Gomb felengedes */
            for (i = 1; menu[i].clicked != -1; i++)
                if (menu[i].toggle == 0) {
                    menu[i].clicked = 0;
                }
            if ((event.key.keysym.sym == SDLK_q || event.key.keysym.sym == SDLK_ESCAPE) && menu[bt_exit].enabled && !textinput) {
                    quit = 1;
                }
            break;
        case SDL_USEREVENT:
            /** Jatekter szele */
            if (proton.x < menuwidth || WIDTH < proton.x || proton.y < 0 || HEIGHT < proton.y) {
                start = -1;
                SDL_RemoveTimer(id1);
                rectangleRGBA(screen, menuwidth, 0, WIDTH, HEIGHT, 255, 0, 0, 255);
                rectangleRGBA(screen, menuwidth+1, 0+1, WIDTH-1, HEIGHT-1, 255, 0, 0, 255);
                SDL_Flip(screen);
                SDL_Delay(1500);
                protonreset = 1;
                menu[bt_mapreset].enabled = 0;
                menu[bt_start].enabled = 0;
            }
            /** Celpont eltalalasa, palya vege */
            else if (celpont_hit(proton, celpontr)) {
                frissit_levelend(screen, &start, levels[currentlevel].resetcounter, levels[currentlevel].pausecounter, currentlevel, menu, bt_level1, &hs_clickable, hs_buttonwidth, highscore_list);
                levels[currentlevel].resetcounter = levels[currentlevel].pausecounter = 0;
            }
            /** Eredo ero kiszamolasa */
            eredovektor.vx = 0;
            eredovektor.vy = 0;
            mozgoptr = elso;
            while (mozgoptr != NULL) {
                eredovektor =  Vektor_osszead(eredovektor, Coulomb_ero(proton, mozgoptr->toltes));
                /** Utkozes toltessel */
                if (ponttav(proton.x, proton.y, (double)mozgoptr->toltes.x, (double)mozgoptr->toltes.y) < toltesr + proton.r) {
                    start = -1;
                    SDL_RemoveTimer(id1);
                    filledCircleRGBA(screen, (mozgoptr->toltes.x + proton.x)/2, (mozgoptr->toltes.y + proton.y)/2, 5, 255, 0, 0, 200);
                    SDL_Flip(screen);
                    SDL_Delay(1500);
                    protonreset = 1;
                    break;
                }
                Toltes_rajzol(screen, mozgoptr->toltes, toltesr, proton, menu[bt_shownumbers].clicked);
                mozgoptr = mozgoptr->kov;
            }
            proton.v = Vektor_osszead(proton.v, eredovektor);

            /** Falhoz eres detektalasa */
            /** Sebesseg maximalizalasa */
            if (fuggoleges_falhoz_er(proton))
                proton.v.vx = 0;
            else
                proton.v.vx = (proton.v.vx < -vmax) ? -vmax : (proton.v.vx > vmax) ? vmax : (proton.v.vx);
            if (vizszintes_falhoz_er(proton))
                proton.v.vy = 0;
            else
                proton.v.vy = (proton.v.vy < -vmax) ? -vmax : (proton.v.vy > vmax) ? vmax : (proton.v.vy);

            /** Mozgatas */
            proton.x += proton.v.vx;
            proton.y += proton.v.vy;
            break;
        case SDL_QUIT:
            quit = 1;
            break;
        }
        /** Kepernyo frissitese */
        if (changelevel == -1) { /** Dicsoseglista megtekintese kozben */
            frissit_hs(screen, highscore_list, hsdb, textinput, &endoftextinput, menu);
        }
        else if (!celpont_hit(proton, celpontr)) /** Normalis mukodes futas kozben */
            frissit(screen, beosztas, elsooszlop, toltesr, proton, levels[currentlevel].resetcounter, levels[currentlevel].pausecounter, menu, bt_shownumbers, celpontr);
        else { /** Palya vegen frissitve tartja a menut es a pontszam kiirast */
            drawmenu(screen, menu);
            if (currentlevel == 5)
                frissit_levelend(screen, &start, levels[currentlevel].resetcounter, levels[currentlevel].pausecounter, currentlevel, menu, bt_level1, &hs_clickable, hs_buttonwidth, highscore_list);
            SDL_Flip(screen);
        }
    }
    /** Memoria felszabaditasa */
    freememory();
    /** Dicsoseglista mentese */
    highscores_write(highscore_list, hsdb);
    /** SDL bezarasa */
    closeSDL();
    printf("\n<--- LOG FILE VEGE --->");
    return 0;
}
