#include <SDL_gfxPrimitives.h>
#include <stdio.h>
#include <math.h>
#include "toltesek.h"

void fatal_error (int errorcode);
extern double t;

/** A toltesek listajanak hasznalatahoz szukseges pointerek. */
ToltesLista *uj;
ToltesLista *mozgoptr;
ToltesLista *elso;
ToltesLista *utolso;
ToltesLista *elsonemfix; /** Az elso nem fix (jatekos altal modosithato) toltesre mutat.   */
ToltesLista *utolsofix; /** Az utolso fix toltesre mutat */


/** Debug fuggveny, egy toltes parametereit irja a logba. */
void Toltes_kiir (Toltes q) {
    printf("x: %d, y: %d, charge: %d\n", q.x, q.y, q.charge);
}

/** Debug fuggveny, a toltesek teljes lancolt listajat irja a logba.*/
void ToltesLista_kiir (void) {
    mozgoptr = elso;
    int i = 1;
    printf("Tolteslistat kezelo toltesek:\n");
    printf("\tuj %p\n\telso %p\n\telsonemfix %p\n\tutolsofix %p\n\tutolso %p\n", uj, elso, elsonemfix, utolsofix, utolso);
    printf("A toltesek lancolt listaja:\n");
    if (mozgoptr == NULL)
        printf("\tA lista jelenleg ures.\n");
    while (mozgoptr != NULL) {
        printf("\t%d. elozo %p, mozgoptr %p, kov %p   ", i, mozgoptr->elozo, mozgoptr, mozgoptr->kov);
        Toltes_kiir(mozgoptr->toltes);
        mozgoptr = mozgoptr->kov;
        i++;
    }
}

/** Kiszamolja a parameterkent kapott toltes es a proton kozott    *
  * fellepo Coulomb-ero nagysagat. A jatek jatszhatosaga erdekeben *
  * ezt nem pontosan a fizikai osszefuggessel teszi, de attol csak *
  * nagysgrendekben tr el. (Egeszen pontosan a "k" konstans      *
  * ertekeben.) Az aranyos fuggesek valtozatlanok maradtak.        */
Vektor Coulomb_ero (Mozgotoltes proton, Toltes q) {
    Vektor v;
    double k = pow(10, 4);
    double rad = asin((proton.y-q.y)/ponttav(proton.x, proton.y, (double)q.x, (double)q.y));
    double vhossz = k * proton.charge * q.charge / pow(ponttav(proton.x, proton.y, (double)q.x, (double)q.y), 2);
    v.vx = vhossz * cos(rad) * t; /** A komponensek t-vel val szorzasa gyorsulasvektort */
    v.vy = vhossz * sin(rad) * t; /** csinal az erovektorbol.                            */
    return v;
}

/** A bemeneti valtozokbol toltes strukturat letrehozo   *
  * fuggveny. Erre azert van szukseg, mert nem mindenutt *
  * hasznalhato a {}-es struktura incializalas.          */
Toltes Toltes_legyen (int x, int y, int charge, int editable) {
    Toltes t = {x, y, charge, editable};
    return t;
}

/** Egy toltest ad hozza a lancolt listahoz, ami a megadott *
  *  parameterekkel rendelkezik. Ez a fuggveny hivodik meg, *
  * ha a jatekos le  akar rakni egy toltest a palyara.      */
void Toltes_letrehoz (int x, int y, int charge) {
    uj = (ToltesLista*)malloc(sizeof(ToltesLista));
    if (uj == NULL) fatal_error(0xA1);
    uj->toltes = Toltes_legyen(x, y, charge, 1);
    uj->kov = NULL;
    uj->elozo = utolso;
    if (elso != NULL) utolso->kov = uj; /** Ha nem az elso elem */
    else { /** Ha az elso elem */
        elso = uj;
        elsonemfix = elso;
    }
    utolso = uj; /** Az uj elem az utolso */
    if (elsonemfix == NULL && utolsofix != NULL)  /** Ha van utolso fix, de nincs elso nem fix toltes */
        elsonemfix = (utolsofix->kov);
}

/** Az elozohoz hasonlo, de attol elteroen ez a palya betoltesekor *
  * a fix tolteseket hozza letre, es adja hozza a listahoz.        */
    void fixToltes_letrehoz (int x, int y, int charge) {
    uj = (ToltesLista*)malloc(sizeof(ToltesLista));
    if (uj == NULL) fatal_error(0xA2);
    uj->toltes = Toltes_legyen(x, y, charge, 0);
    uj->kov = NULL;
    uj->elozo = utolso;
    if (elso != NULL) {
        utolso->kov = uj;
    }
    else
        elso = uj;
    utolso = uj;
    utolsofix = uj;
}

/** Kitorol egy toltest a lancolt  listabol, es felszabaditja *
  * az az altal eddig lefoglalt memoriat.                     */
void Toltes_torol (ToltesLista *prev, ToltesLista *next) {
    if (prev == NULL && next == NULL) { /** Egyetlen toltes torlese */
        elso = NULL;
        elsonemfix = NULL;
        utolso = NULL;
    }
    else if (next == NULL) { /** Utolso toltes trlese */
        prev->kov = NULL;
        utolso = prev;
    }
    else if (prev == NULL) { /** Elso toltes torlese */
        next->elozo = NULL;
        elso = next;
    }
    else { /** Sorban levo toltes torlese */
        prev->kov = next;
        next->elozo = prev;
    }
    free(mozgoptr);
}

/** Megnoveli egy toltes tolteset, vagy ha meg nincs a megadott *
  * helyen toles, akkor letrehoz egyet, "+1" toltessel.         */
void Toltes_novel (Pont racspont) {
    printf("(+) Toltes novelese...\n");
    mozgoptr = elso;
    while (mozgoptr != NULL) {
        if (mozgoptr->toltes.x == racspont.x && mozgoptr->toltes.y == racspont.y) {
            if (mozgoptr->toltes.editable && mozgoptr->toltes.charge < 3) {
                mozgoptr->toltes.charge += 1;
            }
            if (mozgoptr->toltes.charge == 0)
                Toltes_torol(mozgoptr->elozo, mozgoptr->kov);
            ToltesLista_kiir();
            return;
        }
        mozgoptr = mozgoptr->kov;
    }
    Toltes_letrehoz (racspont.x, racspont.y, +1);
    ToltesLista_kiir();
}

/** Csokkenti egy toltes tolteset, vagy ha meg nincs a megadott *
  * helyen toles, akkor letrehoz egyet, "-1" toltessel.         */
void Toltes_csokkent (Pont racspont) {
    printf("(-) Toltes csokkentese...\n");
    mozgoptr = elso;
    while (mozgoptr != NULL) {
        if (mozgoptr->toltes.x == racspont.x && mozgoptr->toltes.y == racspont.y) {
            if (mozgoptr->toltes.editable && mozgoptr->toltes.charge > -3) {
                mozgoptr->toltes.charge -= 1;
            }
            if (mozgoptr->toltes.charge == 0)
                Toltes_torol(mozgoptr->elozo, mozgoptr->kov);
            ToltesLista_kiir();
            return;
        }
        mozgoptr = mozgoptr->kov;
    }
    Toltes_letrehoz (racspont.x, racspont.y, -1);
    ToltesLista_kiir();
}

/** Kirajzol egy toltest a kerpernyore. A toltes szine az *
  * abszolut erteketol  fugg, es a menu gomb allasatol    *
  * fuggoen kiirodik fole a szamerteke is.                */
void Toltes_rajzol (SDL_Surface *screen, Toltes q, int toltesr, Mozgotoltes proton, int shownumbers) {
    /** Szinek */
    int r, g, b, a = 0;
    if (q.charge < 0) {
        r = 0; g = 191; b = 255;
    }
    else { /** (q.charge > 0), mivel 0 nem lehet */
        r = 255; g = 0; b = 0;
    }
    /** Atlatszosag merteke */
    if (!q.editable) {
        a = 125;
        lineRGBA(screen, q.x-sqrt(2)/2*toltesr+1, q.y-sqrt(2)/2*toltesr+1, q.x+sqrt(2)/2*toltesr-1, q.y+sqrt(2)/2*toltesr-1, r, g, b, 250);
        lineRGBA(screen, q.x-sqrt(2)/2*toltesr+1, q.y+sqrt(2)/2*toltesr-1, q.x+sqrt(2)/2*toltesr-1, q.y-sqrt(2)/2*toltesr+1, r, g, b, 250);
    }
    else if (q.editable) {
        switch (abs(q.charge)) {
            case 1: a = 50; break;
            case 2: a = 150; break;
            case 3: a = 255; break;
        }
    }
    /** Kirajzolas (es kiiras) */
    filledCircleRGBA(screen, q.x, q.y, toltesr, r, g, b, a);
    aacircleRGBA(screen, q.x, q.y, toltesr, r, g, b, (int)fmin(a*2, 255));
    if (shownumbers || !q.editable) {
        char chargetext[2];
        sprintf(chargetext, "%+d", q.charge);
        stringRGBA(screen, q.x-8, q.y-20, chargetext, r, g, b, 255);
    }
}

/** Felrajzolja a protont a palyara */
void Mozgotoltes_rajzol (SDL_Surface *screen, Mozgotoltes p) {
    filledCircleRGBA(screen, p.x, p.y, p.r, 255, 255, 255, 255);
    aacircleRGBA(screen, p.x, p.y, p.r, 255, 255, 255, 255);
}
